/*
 *  mapscsi - program for creating consistent scsi device mappings
 *
 *  Author: Michael Clark <michael@metaparadigm.com>
 *  Copyright Metaparadigm Pte. Ltd. 2001
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#ifndef _MAPSCSI_H_
#define _MAPSCSI_H_

#ifndef SCSI_IOCTL_GET_PCI
#define SCSI_IOCTL_GET_PCI 0x5387
#endif

#define MAX_ERRORS 5
#define MAX_SCSI_DEVS 255

#define SCSI_OFF sizeof(struct sg_header)

#define INQUIRY_CMD     0x12
#define INQUIRY_CMDLEN  6
#define INQUIRY_REPLY_LEN 96

#define READ_CMD     0x3c
#define READ_CMDLEN  10
#define READ_REPLY_LEN 256

#define INQUIRY_PERIPHERAL_OFFSET 0
#define INQUIRY_VENDOR_OFFSET  8
#define INQUIRY_VENDOR_LENGTH  8
#define INQUIRY_PRODUCT_OFFSET  16
#define INQUIRY_PRODUCT_LENGTH  16
#define INQUIRY_REVISION_OFFSET  32
#define INQUIRY_REVISION_LENGTH  4
#define INQUIRY_CHANNELID_OFFSET 43
#define INQUIRY_MAXSERIALNO_LENGTH 255

#define HOSTNAME_LEN 64
#define PREFIX_LEN 16
#define PCIINFO_LEN 8
#define PARAM_LEN 32
#define WWN_STR_LEN 16 /* 8 bytes as ascii hex */
#define PORTID_STR_LEN 6 /* 3 bytes as ascii hex */

#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
#define msizeof(TYPE, MEMBER) ((size_t) sizeof(((TYPE *)0)->MEMBER))

typedef struct my_scsi_idlun {
/* why can't userland see this structure ??? */
    int dev_id;
    int host_unique_id;
} My_scsi_idlun;

typedef struct scsi_device {

  int active;
  char hostname[HOSTNAME_LEN+1];
  char pciinfo[PCIINFO_LEN+1];
  char *sg_device;
  char *device;
  char prefix[PREFIX_LEN+1];

  int host;
  int channel;
  int id;
  int lun;
  int part;
  int type;
  int channelid;
  int num_parts;

  /* scsi inquiry device info */
  char vendor[INQUIRY_VENDOR_LENGTH+1];
  char product[INQUIRY_PRODUCT_LENGTH+1];
  char revision[INQUIRY_REVISION_LENGTH+1];
  char serial[INQUIRY_MAXSERIALNO_LENGTH+1];

  /* fibre channel addresses */
  int isfc;
  char wwpn[WWN_STR_LEN+1]; 
  char wwnn[WWN_STR_LEN+1]; 
  char portid[PORTID_STR_LEN+1];
  int loopid;


  struct scsi_device *next;

} scsi_device_t;

typedef struct fc_device {

  char wwpn[WWN_STR_LEN+1];
  char wwnn[WWN_STR_LEN+1];
  char portid[PORTID_STR_LEN+1];
  int loopid;

  int host;
  int channel;
  int id;
  int lun;

  struct fc_device *next;
} fc_device_t;

typedef int (*decode_func_t)(void *dest, const char *str, size_t size);
typedef int (*encode_func_t)(char *str, void *src, size_t size);

typedef struct scsi_device_param {

  char substchar;
  const char *name;
  const char *description;
  encode_func_t enc;
  decode_func_t dec;
  size_t offset;
  size_t size;
  
} scsi_device_param_t;

typedef struct scsi_map_rule {
  int rule_num;
  char *key;
  char *value;
  struct scsi_map_rule *next_constraint;
  struct scsi_map_rule *next_rule;
} scsi_map_rule_t;

typedef struct scsi_map {
  char *device;
  char *link;
  struct scsi_map *next;
} scsi_map_t;

extern scsi_device_t* find_dev_by_loc(int host, int channel, int id, int lun);

#endif
